//@+leo-ver=4
//@+node:@file host/luddev-fops.c
//@@language c
//@<< lud-base #includes >>
//@+node:<< lud-base #includes >>
#ifdef CONFIG_SMP
#define __SMP__
#endif

#include <asm/segment.h>
#include <asm/io.h>
#include <asm/page.h>
#include <asm/semaphore.h>
#include <asm/atomic.h>
#include <asm/system.h>
#include <asm/uaccess.h>
#include <asm/byteorder.h>

#include <linux/module.h>

#include <linux/version.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/errno.h>
#include <linux/pci.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/proc_fs.h>
#include <linux/spinlock.h>
#include <linux/kmod.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
#include <linux/device.h>
#include <linux/cdev.h>
#endif
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,18))
#include <linux/compiler.h>
#else
#define likely(x) (x)
#define unlikely(x) (x)
#endif /* LINUX_VERSION_CODE */

#include <stdarg.h>


#include "compat.h"
#include "lud_mmio.h"
#include "luddite-host.h"
#include "luddite/version.h"
#include "luddite/os.h"
#include "exported.h"

#include "core.h"
#include "procfs.h"
#include "node.h"
#include "helpers.h"
#include "channels.h"

// Define if you want to use the 'standard' linux kernel firmware loading:
//#define USE_FIRMWARE_LOAD
// otherwise we use our own (v. fast) file loading.
#ifdef USE_FIRMWARE_LOAD
#include <linux/firmware.h>
#else
#include <linux/syscalls.h>
static int loadFile(char *file, size_t fileLen, char *dest);
#endif

//@-node:<< lud-base #includes >>
//@nl
//@+others
//@+node:luddevOpen
//@+at
//@nonl
// ************************************************************************ *
// * Luddite device (/dev/luddite/0/lud) file operations           * *
// ************************************************************************
//@-at
//@@c

static int luddevOpen(struct inode * inode,
                  struct file  * file)
{
	int retVal = 0;
	LUD_DEV* m;

#ifdef CONFIG_SYSFS
	m = container_of(inode->i_cdev,LUD_DEV,cdev);
#else
	if (file->private_data) {
		m = (LUD_DEV*)file->private_data;
	} else {
		m = lud_devices[MINOR(inode->i_rdev)].luddev;
	}
#endif
	if (m==NULL) {
		retVal = -ENODEV;
	}
	return retVal;
}
//@-node:luddevOpen
//@+node:luddevRelease

static int luddevRelease(struct inode * inode,
                     struct file  * file)
{
	return 0;
}
//@-node:luddevRelease
//@+node:luddevRead

static ssize_t luddevRead(struct file * file,
                      char * buffer,
                      size_t count,
                      loff_t *ppos)
{
	return -EINVAL;
}
//@-node:luddevRead
//@+node:luddevWrite


static ssize_t luddevWrite(struct file * file,
                      const char * buffer,
                      size_t count,
                      loff_t *ppos)
{
	return -EINVAL;
}
//@-node:luddevWrite
//@+node:luddevIoctl

#define put_user_with_untyped_ptr(variable, destaddress) \
put_user((variable), (__typeof__(&(variable)))destaddress)

static int luddevIoctl(struct inode * inode,
               struct file * file,
               unsigned int cmd,
               unsigned long arg)
{
LUD_DEV* m;
#ifdef CONFIG_SYSFS
m = container_of(inode->i_cdev,LUD_DEV,cdev);
#else
if (file->private_data) {
	m = (LUD_DEV*)file->private_data;
} else {
	m = lud_devices[MINOR(inode->i_rdev)].luddev;
}
#endif
switch (cmd) {
case LUD_CNTL_GET_DRAM_BASE:
{
	return put_user_with_untyped_ptr(m->sdram_phy,arg);
}
case LUD_CNTL_GET_DRAM_LIMIT:
{
	unsigned int limit = m->sdram_phy+m->sdram_size;
	return put_user_with_untyped_ptr(limit,arg);
}
case LUD_CNTL_GET_MMIO_BASE:
{
	return put_user_with_untyped_ptr(m->mmio_phy,arg);
}
case LUD_CNTL_GET_TARGET_FREQ:
{
	return put_user_with_untyped_ptr(m->freq,arg);
}
case LUD_CNTL_GET_TARGET_STATE:
{
	return put_user_with_untyped_ptr(m->state,arg);
}
case LUD_CNTL_RESET:
{
	DEB(1,printk(KERN_DEBUG "Target %d CNTL DOING RESET...\n",m->minor));
	if (m->reset) {
		m->reset(m);
		m->state = LUD_RESET;
	}

	return 0 ;
}
case LUD_CNTL_DEL_NODES:
{
	int i;

	if (down_interruptible(&lud_devices[m->minor].mod_sem)) {
		return -ERESTARTSYS;
	}

	for (i=lud_devices[m->minor].loader->major+1;i<LUD_MAX_NODES;i++) {
		del_node(m->minor,i);
	}
	cleanstr(m->minor);
	if (m->init)
		m->init(m);
	m->state = 0;
	initstr(m->minor);

	up(&lud_devices[m->minor].mod_sem);

	return 0;
}
case LUD_CNTL_SET_DEBUG_EVENT:
{
	lud_send_command(lud_devices[m->minor].loader,LUD_SYS_SET_DEBUG_EVENT,arg,0,0,0);
	return 0;
}
case LUD_CNTL_GET_DEBUG_EVENT:
{
	unsigned int ret = lud_send_request(lud_devices[m->minor].loader,LUD_SYS_GET_DEBUG_EVENT,0,0,0,0);
	return put_user_with_untyped_ptr(ret,arg);
}
case LUD_CNTL_SET_DP_LEVEL:
{
	lud_devices[MINOR(inode->i_rdev)].dprintk_level = arg;
	lud_send_command(lud_devices[m->minor].loader,LUD_SYS_PRINTK,arg,0,0,0);
	return 0;
}
case LUD_CNTL_SET_COMM_VEC:
{
	unsigned long pci_addr = ((arg+63)& (~63));
	LUD_COMVEC* c = (LUD_COMVEC*) lud_remap_addr(m,pci_addr);
	m->comvec = pci_addr;
	m->host_comm_vec = (unsigned long)&c[0].major;
	m->target_comm_vec = (unsigned long)&c[1+m->minor].major;
	m->host_comm_addr = (unsigned long)&c[0].com;
	m->target_comm_addr = (unsigned long)&c[1+m->minor].com;
	return 0;
}
case LUD_CNTL_GET_DP_LEVEL:
{
	return put_user_with_untyped_ptr(lud_devices[MINOR(inode->i_rdev)].dprintk_level, arg);
}
case LUD_CNTL_START:
{
	DEB(1,printk(KERN_DEBUG "Target %d ARrrrggghhhh here goes....\n",m->minor));
	if (m->start) {
		m->start(m);
		m->state = LUD_RUNNING;
	}
	return 0 ;
}
case LUD_CNTL_START_LUDDITE:
{
	DEB(1,printk(KERN_DEBUG "Target %d ARrrrggghhhh here goes....\n",m->minor));
	if (m->start) {
		m->start(m);
		m->state = LUD_RUNNING;
	}
	interruptible_sleep_on_timeout(&m->start_queue,2*HZ);
	if (m->state == LUD_LUDDITE)
		return 0 ;
	return -ENODEV;
}
case LUD_CNTL_DL_MEMIMAGE:
{
	/* Downloads a TriMedia Memory Image (Created with
	   the tmld -mi option, to the Target. This *MUST* be
	   located at the correct memory position */
	tmMI_t * mi = (tmMI_t *) arg ;
	unsigned int len ;
	unsigned int nameLen ;
	char name[255];
	DEB(1,printk(KERN_DEBUG "Target %d download image\n",m->minor));
	if (unlikely(!access_ok(VERIFY_READ, mi, sizeof(*mi)))
	 || unlikely(__get_user(nameLen, &(mi->nameLength)))
	 || unlikely(copy_from_user(name, mi->name, nameLen + 1))
	 || unlikely(__get_user(len, &(mi->length))))
		return -EFAULT ;

	if (clear_target) {
		DEB(1,printk(KERN_DEBUG "TM %d clearing memory %p - %p\n", m->minor,(void*)m->sdram_log, (void*)((unsigned long)m->sdram_log + m->sdram_size)));
		memset_io((void*)m->sdram_log, 0, m->sdram_size);
	}

	DEB(1,printk(KERN_DEBUG "Target %d download image size %u file %s addr 0x%08lx\n",
	    m->minor, len, name, m->sdram_log));

	if (len > m->sdram_size) {
		printk(KERN_ERR "Target Cntl: Cannot load memory image, larger than SDRAM. Size = 0x%08x\n", len) ;
		return -EINVAL ;
	}

#ifdef USE_FIRMWARE_LOAD
	{
	char *basename;
	struct firmware firmware;
	firmware.size = len;
	firmware.data = (u8*)m->sdram_log;
	basename = strrchr(name, '/') + 1;
	return request_firmware_known_size(&firmware, len, basename, &m->ldev);
	}
#else
	return loadFile(name, len, (char*)m->sdram_log);
#endif
}
case LUD_CNTL_LIST_DLL:
{
	int i;
	int l = 0;
	int ret;

	struct LUD_MODLIST* tml = (struct LUD_MODLIST*)arg;

	if (down_interruptible(&lud_devices[m->minor].mod_sem)) {
		return -ERESTARTSYS;
	}

	DEB(1,printk(KERN_DEBUG "Target %d list dll\n",m->minor));
		for (i=0;i<LUDMOD_MAX_MODULES;i++) {
			if (lud_devices[m->minor].lud_modules[i].name) {
				const char * name = lud_devices[m->minor].lud_modules[i].name;
				int len = strlen(name);
				if (len>=LUDMOD_MAX_MOD_NAME) {
					len = LUDMOD_MAX_MOD_NAME-1;
				}
				if (unlikely(copy_to_user(&(tml->mod[l][0]),name,len+1))) {
					up(&lud_devices[m->minor].mod_sem);
					return -EFAULT;
				}
			l++;
		}
	}
	ret = unlikely(copy_to_user(&tml->n,&l,sizeof(l))) ? -EFAULT : 0;
	up(&lud_devices[m->minor].mod_sem);

	return ret;
}
case LUD_CNTL_UNLOAD_DLL:
{
	int i=0;
	int ret;
	char p[128];
	char* mname = (char*)arg;
	if (m->state != LUD_LUDDITE ) {
		return -ENODEV;
	}

	if (down_interruptible(&lud_devices[m->minor].mod_sem)) {
		return -ERESTARTSYS;
	}

	do {
		if (unlikely(copy_from_user(&p[i],mname,1))) {
			up(&lud_devices[m->minor].mod_sem);
			return -EFAULT;
		}
		mname++;
		i++;
	} while ((p[i-1]!=0) && (i<128));
	p[i-1] = 0;

	DEB(1,printk(KERN_DEBUG "Target %d unload dll %s\n",m->minor,mname));

	up(&lud_devices[m->minor].mod_sem);
	ret =  uninstall_mod(m->minor,p);

	return ret;
}
case LUD_CNTL_LOAD_DLL:
{
	int status;
	char* name;
	char* mname;
	char* ptr;
	char* opt;
	char* opts;
	unsigned int laddr_tm;
	unsigned long laddr;
	char* scratch;
	char* tmp;
	long len;
	int slen;
	long pri;
	int string_length;
	if (m->state != LUD_LUDDITE ) {
		printk(KERN_CRIT "lud-core: LUD_CNTL_LOAD_DLL failed, TriMedia not running Luddite\n");
		return -ENODEV;
	}

	if (down_interruptible(&lud_devices[m->minor].mod_sem)) {
		return -ERESTARTSYS;
	}

	if (get_user(mname,(char**)arg)
	 || get_user(ptr,(char**)(arg+sizeof(mname)))
	 || get_user(len,(long*)(arg+sizeof(mname)+sizeof(ptr)))
	 || get_user(opt,(char**)(arg+sizeof(mname)+sizeof(ptr)+sizeof(len)))
	 || get_user(pri,(long*)(arg+sizeof(mname)+sizeof(ptr)+sizeof(len)+sizeof(opt)))) {
		up(&lud_devices[m->minor].mod_sem);
		return -EFAULT;
	}

	DEB(1,printk(KERN_DEBUG "Target %d load dll\n",m->minor));

	name = kmalloc(128,GFP_KERNEL);
	if (unlikely(!name)) {
		up(&lud_devices[m->minor].mod_sem);
		return -ENOMEM;
	}
	opts = kmalloc(128,GFP_KERNEL);
	if (unlikely(!opts)) {
		kfree(name);
		up(&lud_devices[m->minor].mod_sem);
		return -ENOMEM;
	}

	slen = 128;

	string_length = strncpy_from_user(name, mname, slen);
	if (unlikely(string_length >= slen)) {
		/* String too long */
		string_length = -EINVAL; /* FIXME correct error code? */
	}
	if (unlikely(string_length < 0)) {
		kfree(name);
		kfree(opts);
		up(&lud_devices[m->minor].mod_sem);
		return string_length; /* == -EFAULT or as set above*/
	}

	scratch = (char*)m->scratch;
	strcpy_toio(scratch,name);

	tmp = &scratch[strlen(name)+1];
	slen -= (strlen(name)+1);

	string_length = strncpy_from_user(opts, opt, slen);
	if (unlikely(string_length >= slen)) {
		/* String too long */
		string_length = -EINVAL; /* FIXME correct error code? */
	}
	if (unlikely(string_length < 0)) {
		kfree(name);
		kfree(opts);
		up(&lud_devices[m->minor].mod_sem);
		return string_length; /* == -EFAULT or as set above*/
	}

	strcpy_toio(tmp,opts);

	laddr_tm = lud_send_request(lud_devices[m->minor].loader,LUD_SYS_PRELOAD,len,0,0,0);
	laddr = lud_remap_addr(m,laddr_tm);
	if (laddr==0) {
		printk(KERN_EMERG "Bad pointer from Target 0x%x\n",laddr_tm);
		kfree(name);
		kfree(opts);
		up(&lud_devices[m->minor].mod_sem);
		return -ENOMEM;
	}

	if (unlikely(copy_from_user_toio((void*)laddr,ptr,len))) {
		/* FIXME: cleanup needed on Target? */
		kfree(name);
		kfree(opts);
		up(&lud_devices[m->minor].mod_sem);
		return -EFAULT;
	}

	if (pri<1 || pri>250)
		pri = LUD_DEF_PRI;

	status = lud_send_request(lud_devices[m->minor].loader,LUD_SYS_LOAD,pri,0,0,0);

	switch (status) {
	case 0:
	case 1:
		kfree(name);
		kfree(opts);
		up(&lud_devices[m->minor].mod_sem);
		return -EINVAL;
	case LUD_LOAD_BUSY:
		kfree(name);
		kfree(opts);
		up(&lud_devices[m->minor].mod_sem);
		return -EEXIST;
	case LUD_LOAD_SYSERROR:
		kfree(name);
		kfree(opts);
		up(&lud_devices[m->minor].mod_sem);
		return -EBADF;
	case LUD_LOAD_LOGICERROR:
		kfree(name);
		kfree(opts);
		up(&lud_devices[m->minor].mod_sem);
		return -EPERM;
	case LUD_LOAD_FILEERROR:
		kfree(name);
		kfree(opts);
		up(&lud_devices[m->minor].mod_sem);
		return -ENOEXEC;
	case LUD_LOAD_INITERROR:
		kfree(name);
		kfree(opts);
		up(&lud_devices[m->minor].mod_sem);
		return -EIO;
	default:
		install_mod(m->minor,name,opts,pri,status);
	}
	up(&lud_devices[m->minor].mod_sem);
	return 0;
}
case LUD_CNTL_GET_ID:
{
	return unlikely(copy_to_user((void*)arg,m->name,strlen(m->name)+1)) ? -EFAULT : 0;
}
case LUD_CNTL_GET_TYPE:
{
	return put_user_with_untyped_ptr(m->lud_type, arg);
}
case LUD_CNTL_GET_MMIO_REG:
{
	tmMMIOReg_t * mmioReg = (tmMMIOReg_t *) arg ;
	unsigned int temp ;

	if (unlikely(!access_ok(VERIFY_WRITE, mmioReg, sizeof(*mmioReg)))
	 || unlikely(__get_user(temp, &(mmioReg->reg))))
		return -EFAULT ;

	temp = MMIOread(m,temp) ;

	if (unlikely(__put_user(temp, &(mmioReg->val))))
		return -EFAULT ;
	return 0 ;
}
case LUD_CNTL_GET_DRAM_WORD:
{
	tmDRAMAccess_t * dramAcc = (tmDRAMAccess_t *) arg ;
	unsigned int temp ;

	if (unlikely(!access_ok(VERIFY_WRITE, dramAcc, sizeof(*dramAcc)))
	 || unlikely(__get_user(temp, &(dramAcc->pos))))
		return -EFAULT ;

	temp = *((unsigned long *)(m->sdram_log + temp)) ;

	if (unlikely(__put_user(__le32_to_cpu(temp), &(dramAcc->val))))
		return -EFAULT ;

	return 0 ;
}
case LUD_CNTL_SET_MMIO_REG:
{
	unsigned int reg, val ;
	tmMMIOReg_t * mmioReg = (tmMMIOReg_t *) arg ;
	if (unlikely(!access_ok(VERIFY_READ, mmioReg, sizeof(*mmioReg)))
	 || unlikely(__get_user(val, &(mmioReg->val)))
	 || unlikely(__get_user(reg, &(mmioReg->reg))))
		return -EFAULT ;

	MMIOwrite(val,m,reg);
	return 0 ;
}
case LUD_CNTL_SET_DRAM_WORD:
{
	tmDRAMAccess_t * dramAcc = (tmDRAMAccess_t *) arg ;
	unsigned int val, addr ;

	if (unlikely(!access_ok(VERIFY_READ, dramAcc, sizeof(*dramAcc)))
	 || unlikely(__get_user(val, &(dramAcc->val)))
	 || unlikely(__get_user(addr, &(dramAcc->pos))))
	 	return -EFAULT;

	*((unsigned long *)(m->sdram_log + addr)) = __cpu_to_le32(val);

	return 0 ;
}
default:
	printk(KERN_DEBUG "Target %d : Unknown IOCTL Command %d, Arg 0x%08lx\n",m->minor,
	   cmd, arg) ;
	return -EINVAL ;
}

return -EINVAL ;
}


struct file_operations lud_dev_fops = {
read:		luddevRead,
write:		luddevWrite,
ioctl:		luddevIoctl,
open:		luddevOpen,
release:	luddevRelease,
} ;

#ifndef USE_FIRMWARE_LOAD
// Load file into memory.
static int loadFile(char *file, size_t fileLen, char *dest)
{
    int st = -EIO;
    int fd = -1;
    size_t len;
    mm_segment_t saved_addr_limit;

    /* This workaround (calling set_fs(KERNEL_DS)) prevent the memory access checks
       that normally would prevent kernel space code calling 'user-space' functions,
       such as open, read, write, close.
    */
    saved_addr_limit = get_fs();
    set_fs(KERNEL_DS);

    fd = sys_open(file, O_RDONLY, 0);
    if (fd < 0)
    {
        st = fd;
        printk(KERN_ERR "error %d opening %s\n", st, file);
        goto DEPARTURES;
    }

    len = sys_read(fd, dest, fileLen);
    if (len != fileLen)
    {
        printk(KERN_ERR "sys_read from %s %u bytes=>%u\n", file, fileLen, len);
    }
    else
    {
        st = 0;
    }

  DEPARTURES:
    if (fd >= 0)
    {
        sys_close(fd);
    }
    set_fs(saved_addr_limit);

    return st;
}
#endif

//@-node:luddevIoctl
//@-others
//@-node:@file host/luddev-fops.c
//@-leo
