#ifndef PHSTBRPC_COMMON_H    /* Multi-include protection.   <FILENAME>_H */
#define PHSTBRPC_COMMON_H

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright (C) 2006 NXP B.V.
 * All Rights Reserved.
*//**
    @file   phStbRpc_Common.h
    @brief  RPC common definitions.

@b  Component: phStbRpc.

*//*

Rev Date        Author      Comments
--------------------------------------------------------------------------------
001 20050418    batelaan    Original
002 20050506    batelaan    Various.
003 20050512    batelaan    Crosscheck fixes.
                            Remove need for limits.h (problem with kernel build).
004 20050622    batelaan    Rename phStbRpc_Option_None to phStbRpc_Option_Wait
005 20050623    mpage       Added MPEG related handler IDs.
006 20050627    mpage       Added Demux handler ID.
007 20050627    mpage       Fixing name for Demux handler ID.
008 20050629    Mike Neill  Added Jpeg handler ID.
009 20050629    Mike Neill  Added Png handler ID.
010 20050630    mpage       Added Audio Renderer handler ID.
011 20050706    mpage       Added FileRead and FBRead.
012 20050711    Mike Neill  Added Gif handler ID.
013 20050711    Mike Neill  Added Bmp handler ID.
014 20050713    laird       Add kernel handler IDs.
015 20050725    mpage       Add Windows Media handler IDs.
015.1.1 20050725    mpage   Add MP4Read handler ID.
016 20050804    raiyat      Unify still image decoders.
017 20050809    raiyat      Merge with 15.1.1
15.1.2 20050809  burningh   Add Event channel IDs
18  20050810    raiyat      Merge with 15.1.2
19  20050810    mpage       Adding more IDs
20  20050810    mpage       Putting the old image IDs back so that the build
                            still works.
21  20050811    raiyat      Remove old image decoders IDs
21.1.1 20050816 clement     Add Ac3 handler ID.
22  20050817    burningh    Add DemuxTs handler ID
23  20050931    laird       Adding KFBread and userspace FBread handles.
24  20050908    batelaan    Enable second TM.
25  20050919    clement     Add SPDIF handler ID.
26  20051004    clement     Add TEE/Splitter component handler ID.
27  20051102    batelaan    Add handler names; remove kernel mode handlers.
28  20051116    mpage       Add phStbDspDataTransfer.
29  20060104    laird       Adding Custom handler ids for use by customers.
30  20060116    neill       Add analog input
31  20060117    burningh    Add Fwrite
32  20060124    batelaan    Add extern "C" for use with C++
33  20060124    mpage       Add EsRead
34  20060208    batelaan    Corrected PH_STBRPC_HANDLERID_NAMES
35  20060210    burningh    Add H264Vid
36  20060505    mpage       Add more event channels
37  20060724    burningh    Add AvEncoder
37.1.1 20070509 mpage       Add GfxScaler
38  20070504    batelaan    Add TrickMode
39  20070503    bassett     Add FlashPlayer
40  20070704    laird       Add DspMemShCached
41  20070711    mpage       Add PcmAud
42  20070730    laird       Updated Header
43  20070818    mpage       Added StreamRepository
--------------------------------------------------------------------------------
    For consistency and standardisation retain the Section Separators.
*/


/*******************
* INCLUDE FILES    *
********************/

#include <phStbCommon.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**************************************************************
* EXPORTED MACROS         PH_STBRPC_<_WORD>+                  *
***************************************************************/

/** Maximum name length of a luddite node.
    Since we use "rpcXXX" with XXX being an integer 12 should be plenty. */
#define PH_STBRPC_LUD_NODE_NAME_LEN 12U

/** Very large maximum payload size in bytes of an RPC call.
    In practice limited by maxPayloadSize argument
    in phStbRpcTM_HandlerCreate().
*/
#define PH_STBRPC_RPCBUF_PAYLOAD_MAX_SIZE 0x10000000U

/** Overhead of an ::phStbRpc_RpcBuf_t (excludes the payload buffer). */
#define PH_STBRPC_RPCBUF_OVERHEAD \
    (sizeof(phStbRpc_RpcBuf_t) - PH_STBRPC_RPCBUF_PAYLOAD_MAX_SIZE)

/** Actual payload size in bytes of an RPC call. */
#define PH_STBRPC_RPCBUF_PAYLOAD_SIZE(pRpcBuf) \
    ((pRpcBuf)->size - PH_STBRPC_RPCBUF_OVERHEAD)

/** Actual RPC buffer size in bytes for a given payload size. */
#define PH_STBRPC_RPCBUF_SIZE(payloadSize) \
    (payloadSize + PH_STBRPC_RPCBUF_OVERHEAD)

/** Bitshift of target cpu index to a ::phStbRpc_TargetId_t. */
#define PH_STBRPC_TARGETID_SHIFT 16U

/** Form a handlerId with non-default targetId. */
#define PH_STBRPC_HANDLERID(targetId, handlerId) ((handlerId) | ((targetId) << PH_STBRPC_TARGETID_SHIFT))

#define PH_STBRPC_TARGETID_MASK ((~0U) << PH_STBRPC_TARGETID_SHIFT)

#define PH_STBRPC_GET_TARGETID(handlerId) ((handlerId) >> PH_STBRPC_TARGETID_SHIFT)

#define PH_STBRPC_GET_HANDLERID(handlerId) ((handlerId) & ~PH_STBRPC_TARGETID_MASK)

#define PH_STBRPC_OK_TARGETID(_targetId) \
    ((_targetId) >= phStbRpc_TargetId_First && \
     (_targetId) <= phStbRpc_TargetId_Last)

#define PH_STBRPC_OK_HANDLERID(_handlerId) \
    ((_handlerId) >= 0 &&                             \
     (_handlerId) < phStbRpc_HandlerId_Count)

#define PH_STBRPC_OK_TARGETANDHANDLERID(_targetId, _handlerId) \
    (PH_STBRPC_OK_TARGETID(_targetId) && \
     PH_STBRPC_OK_HANDLERID(_handlerId))

/* Names of RPC handlers.
   @see gphStbRpc_HandlerId_t
   @see gphStbRpc_HandlerNames
*/
#define PH_STBRPC_HANDLERID_NAMES \
    "Internal", \
    "Event", \
    "File", \
    "VideoLayer", \
    "VideoRenderer", \
    "VideoScaler", \
    "StreamingSystem", \
    "MpegVid", \
    "MpegAud", \
    "Mpeg4Vid", \
    "Mpeg4Aud", \
    "DemuxRes", \
    "Image", \
    "AudioRenderer", \
    "FileRead", \
    "WMTMgr", \
    "WMVDec", \
    "WMADec", \
    "Ac3Aud", \
    "MP4Read", \
    "Event1", \
    "Event2", \
    "Event3", \
    "Event4", \
    "Event5", \
    "Event6", \
    "Event7", \
    "Event8", \
    "Event9", \
    "Event10", \
    "Event11", \
    "Event12", \
    "Event13", \
    "Event14", \
    "Event15", \
    "DemuxPS", \
    "AviRead", \
    "DivxVid", \
    "DemuxTS", \
    "FBRead", \
    "SpdifRenderer", \
    "Tee", \
    "DspDataTransfer", \
    "AnaVid", \
    "AnaAud", \
    "FileWrite", \
    "EsRead", \
    "H264Vid", \
    "DspAvEncoder", \
    "GfxScaler", \
    "TrickMode", \
    "FlashPlayer", \
	"SharedDspMem", \
    "PcmAud", \
    "StreamRepository", \
    "Custom1", \
    "Custom2", \
    "Custom3", \
    "Custom4", \
    "Custom5", \
    "Custom6", \
    "Custom7", \
    "Custom8"

/**************************************************************
* EXPORTED TYPEDEFS       [p]phStbRpc_<Word>+_t               *
***************************************************************/

/** Options for individual RPC functions.
    @see phStbRpc_RpcCall() ::phStbRpcTM_RpcEntry_t
*/
typedef enum phStbRpc_Option {
    /** Normal synchronous RPC function that waits for its results. */
    phStbRpc_Option_Wait = 1<<0,

    /** Function does not expect to return any data, and will not wait for results. */
    phStbRpc_Option_Async = 1<<1,

    /** Rpc data resides in userspace, not kernel space. */
    phStbRpc_Option_UserSpace = 1<<2
} phStbRpc_Option_t;

/** IDs of RPC targets, i.e. remote CPUs. The default is ::phStbRpc_TargetId_0. */
typedef enum phStbRpc_TargetId {
    phStbRpc_TargetId_First = 0,
    phStbRpc_TargetId_0 = 0,
    phStbRpc_TargetId_1,
    phStbRpc_TargetId_Last = 1
} phStbRpc_TargetId_t;

/** IDs of RPC handlers. Each handler represents a RPC handler task
    on Trimedia.
    Concurrent RPC calls are only possible if they use different handlers.
    @note KEEP #PH_STBRPC_HANDLERID_NAMES UP-TO-DATE!!!
    @see PH_STBRPC_HANDLERID_NAMES
    @see gphStbRpc_HandlerNames
    @see PH_STBRPC_OK_HANDLERID
*/
typedef enum phStbRpc_HandlerId {

    /** For use by phStbRpc. */
    phStbRpc_HandlerId_Internal = 0,

    /** For use by phStbEvent. */
    phStbRpc_HandlerId_Event,

    /** For use by phStbFile. */
    phStbRpc_HandlerId_File,

    /** For use by phStbVideoLayer. */
    phStbRpc_HandlerId_VideoLayer,

    /** For use by phStbVideoRenderer. */
    phStbRpc_HandlerId_VideoRenderer,

    /** For use by phStbVideoScaler. */
    phStbRpc_HandlerId_VideoScaler,

    /** For use by phStbStreamingSystem. */
    phStbRpc_HandlerId_StreamingSystem,

    /** For use by phStbMpegVid. */
    phStbRpc_HandlerId_MpegVid,

    /** For use by phStbMpegAud. */
    phStbRpc_HandlerId_MpegAud,

    /** For use by phStbMpegVid. */
    phStbRpc_HandlerId_Mpeg4Vid,

    /** For use by phStbMpegAud. */
    phStbRpc_HandlerId_Mpeg4Aud,

    /** For use by phStbDemuxRes. */
    phStbRpc_HandlerId_DemuxRes,

    /** For use by phStbImage. */
    phStbRpc_HandlerId_Image,

    /** For use by phStbAudioRenderer. */
    phStbRpc_HandlerId_AudioRenderer,

    /** For use by phStbFileRead. */
    phStbRpc_HandlerId_FileRead,

    /** For use by phStbWMTMgr. */
    phStbRpc_HandlerId_WMTMgr,

    /** For use by phStbWMVDec. */
    phStbRpc_HandlerId_WMVDec,

    /** For use by phStbWMADec. */
    phStbRpc_HandlerId_WMADec,

    /** For use by phStbAc3Aud. */
    phStbRpc_HandlerId_Ac3Aud,

    /** For use by phStbMP4Read. */
    phStbRpc_HandlerId_MP4Read,

    /** For use by phStbEvent channels - DO NOT separate these.
     *  To increase the number, add more _EventChannelX BEFORE _EventChannelMax */
    phStbRpc_HandlerId_EventChannelMin,
    phStbRpc_HandlerId_EventChannel2,
    phStbRpc_HandlerId_EventChannel3,
    phStbRpc_HandlerId_EventChannel4,
    phStbRpc_HandlerId_EventChannel5,
    phStbRpc_HandlerId_EventChannel6,
    phStbRpc_HandlerId_EventChannel7,
    phStbRpc_HandlerId_EventChannel8,
    phStbRpc_HandlerId_EventChannel9,
    phStbRpc_HandlerId_EventChannel10,
    phStbRpc_HandlerId_EventChannel11,
    phStbRpc_HandlerId_EventChannel12,
    phStbRpc_HandlerId_EventChannel13,
    phStbRpc_HandlerId_EventChannel14,
    phStbRpc_HandlerId_EventChannelMax,

    /** For use by phStbDemuxPS. */
    phStbRpc_HandlerId_DemuxPS,

    /** For use by phStbAviRead. */
    phStbRpc_HandlerId_AviRead,

    /** For use by phStbDivxVid. */
    phStbRpc_HandlerId_DivxVid,

    /** For use by phStbDemuxTS. */
    phStbRpc_HandlerId_DemuxTS,

    /** For use by FBread user space module. */
    phStbRpc_HandlerId_FBRead,

    /** For use by phStbSpdifRenderer. */
    phStbRpc_HandlerId_SpdifRenderer,

    /** For use by phStbTee. */
    phStbRpc_HandlerId_Tee,

    /** For use by phStbDspDataTransfer. */
    phStbRpc_HandlerId_DspDataTransfer,

    /** For use by phStbAnaVid. */
    phStbRpc_HandlerId_AnaVid,

    /** For use by phStbAnaAud. */
    phStbRpc_HandlerId_AnaAud,

    /** For use by phStbFileWrite. */
    phStbRpc_HandlerId_FileWrite,

    /** For use by phStbEsRead. */
    phStbRpc_HandlerId_EsRead,

    /** For use by phStbH264Vid. */
    phStbRpc_HandlerId_H264Vid,

    /** For use by phStbDspAvEncoder. */
    phStbRpc_HandlerId_DspAvEncoder,

    /** For use by phStbGfxScaler. */
    phStbRpc_HandlerId_GfxScaler,

    /** See phStbTrickMode_RpcInit(). */
    phStbRpc_HandlerId_TrickMode,

    /* For use by phStbFlashPlayer */
    phStbRpc_HandlerId_FlashPlayer,

	/* For use by phStbDspMemShCached */
	phStbRpc_HandlerId_DspMemShCached,

    /* For use by phStbPcmAud */
	phStbRpc_HandlerId_PcmAud,

    /* For use by phStbStreamRepository */
    phStbRpc_HandlerId_StreamRepository,

    /* !!! NOTE: keep #define PH_STBRPC_HANDLERID_NAMES defined above UP-TO-DATE!!! */




    /** For use by Customers - DO NOT separate these.*/
    phStbRpc_HandlerId_Custom1,
    phStbRpc_HandlerId_Custom2,
    phStbRpc_HandlerId_Custom3,
    phStbRpc_HandlerId_Custom4,
    phStbRpc_HandlerId_Custom5,
    phStbRpc_HandlerId_Custom6,
    phStbRpc_HandlerId_Custom7,
    phStbRpc_HandlerId_Custom8,

    /** Not a handlerId, but the number of values of phStbRpc_HandlerId_t. */
    phStbRpc_HandlerId_Count

} phStbRpc_HandlerId_t;

/** Id of an Rpc function. Normally values are created by declaring
    an enum type. */
typedef UInt32 phStbRpc_FunId_t;

/** Actual information structure that is transferred with each Rpc call.

    Warning: this structure layout must be identical on MIPS/Linux and TM/pSOS.
    Use of UInt32 is be safe given that the endianness is identical.
    @see phStbRpc_RcpCall().
*/
typedef struct phStbRpc_RpcBuf {

    /** Actual runtime byte size of this structure. */
    UInt32 size;

    /** Id of this Rpc handler. */
    UInt32 handlerId;

    /** - On transfer from host to target: Id of this Rpc function being called.
        - On return from target to host: return value of the actual target function call.
        This value is returned using the pResult parameter of the phStbRpc_RpcCall() function. */
    UInt32 funIdOrResult;

    /** - On transfer from host to target: flags for this RPC call (see ::phStbRpc_Option_t).
        - On return from target to host: return RPC status; 0 if the function has been called,
        otherwise an error code meaning that the RPC mechanism failed.
        This value is returned from the phStbRpc_RpcCall() function. */
    UInt32 rpcFlagsOrStatus;

    /** Payload data representing the arguments and return data of the Rpc function.
        The actual payload data length is given by
        the payloadMaxSize argument of ::phStbRpc_HandlerCreate for this RPC channel.
        @see PH_STBRPC_RPCBUF_OVERHEAD PH_STBRPC_RPCBUF_PAYLOAD_SIZE PH_STBRPC_RPCBUF_SIZE
    */
    UInt8 buffer[PH_STBRPC_RPCBUF_PAYLOAD_MAX_SIZE];

} phStbRpc_RpcBuf_t;

/** Pointer to a ::phStbRpc_RpcBuf_t. */
typedef phStbRpc_RpcBuf_t *pphStbRpc_RpcBuf_t;


/*******************************************************************
* EXPORTED DATA      g[k|p|kp|pk|kpk]phStbRpc_<Word>+              *
********************************************************************/
/** Array of names of handlers. Indexed by ::phStbRpc_HandlerId_t.
    @see PH_STBRPC_HANDLERID_NAMES */
extern const char *gphStbRpc_HandlerNames[];

/*******************************************************************
* EXPORTED FUNCTION PROTOTYPES     phStbRpc[_<Word>+]              *
********************************************************************/

#ifdef __cplusplus
}
#endif

#endif /* PHSTBRPC_COMMON_H      Do not add any thing below this line */
