#ifndef PHSTBRPC_MODULE_H    /* Multi-include protection.   <FILENAME>_H */
#define PHSTBRPC_MODULE_H

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright (C) 2006 NXP B.V.
 * All Rights Reserved.
*//**
    @file   phStbRpc_Module.h
    @brief  Header file for implementation of kernel-mode Rpc Xfer interface.

@b  Component: phStbRpc.

*//*

Rev Date        Author      Comments
--------------------------------------------------------------------------------
  1 20051109    batelaan    Original
1.1.1 20070730  laird       Updated Header
--------------------------------------------------------------------------------
    For consistency and standardisation retain the Section Separators.
*/


/*******************
* INCLUDE FILES    *
********************/

#if (defined(USE_LUDDITE) && defined(USE_TMMAN)) || !(defined(USE_LUDDITE) || defined(USE_TMMAN))
  #error Exactly one of USE_LUDDITE or USE_TMMAN should be defined
#endif

#ifdef USE_LUDDITE
  #include "phStbRpc_LudditeXferModule.h"
#endif
#ifdef USE_TMMAN
  #include "phStbRpc_TmmanXferModule.h"
#endif

/**************************************************************
* EXPORTED MACROS         PH[<LNX|TM|NONE>]_IPSTBRPC_<_WORD>+        *
***************************************************************/

/** Value of ::phStbRpc_Xfer lastFunctionId field if the Xfer
    channel has never been initialised.
    @see phStbRpc_Xfer_InitHandler
    @see phStbRpc_Xfer_Lookup
*/
#define PH_STBRPC_XFER_UNINITIALISED (-2)

/** Value of ::phStbRpc_Xfer lastFunctionId field if the Xfer
    channel has been initialised, but no function yet has been called.
    @see phStbRpc_Xfer_InitHandler
    @see phStbRpc_Xfer_Lookup
*/
#define PH_STBRPC_XFER_INITIALISED (-1)

/**************************************************************
* EXPORTED TYPEDEFS       [p]ph[<lnx|tm|NONE>]StbRpc_<Word>+_t  *
***************************************************************/

/** Linux kernel mode data for transferring data to/from target CPU.
    The macro PH_STBRPC_XFER_MEMBERS defines the fields that a
    specific (Luddite/TMMAN) implementation needs.
    @see phStbRpc_Xfer_t
    @see PH_STBRPC_XFER_MEMBERS
    */
struct phStbRpc_Xfer {
    UInt32 targetId;
    UInt32 handlerId;
    struct semaphore mutex;
    UInt32 nCalls;
    /** Last function called, or:
        - #PH_STBRPC_XFER_UNINITIALISED
        - #PH_STBRPC_XFER_INITIALISED
    */
    Int lastFunctionId;
    PH_STBRPC_XFER_MEMBERS;
};

/*******************************************************************
* EXPORTED DATA      g[k|p|kp|pk|kpk]ph[<lnx|tm|NONE>]StbRpc_<Word>+ *
********************************************************************/

/*******************************************************************
* EXPORTED FUNCTION PROTOTYPES     ph[<lnx|tm|NONE>]StbRpc[_<Word>+]   *
********************************************************************/

/* Lookup and initialise (see phStbRpc_Xfer_InitHandler()) if needed an Xfer structure.
   After this function the Xfer mutex is locked.
   @return Error code on error (mutex will be unlocked), otherwise TM_OK.
 */
tmErrorCode_t phStbRpc_Xfer_Lookup(
    UInt32 targetId,
    UInt32 handlerId,
    pphStbRpc_Xfer_t *ppXfer);

/* Initialise an Rpc handler structure for active RPC use.
   This may mean allocating shared memory structures.
   @return Error code on error, otherwise TM_OK.
 */
tmErrorCode_t phStbRpc_Xfer_InitHandler(
    pphStbRpc_Xfer_t pXfer,
    phStbRpc_TargetId_t targetId,
    phStbRpc_HandlerId_t handlerId);

#endif /* PHSTBRPC_MODULE_H      Do not add any thing below this line */
